@@ -64,7 +64,7 @@
 #include	"maildir/maildiraclt.h"
 #include	"maildir/maildirnewshared.h"
 #include	"maildir/maildirinfo.h"
-#include	<unicode.h>
+#include	<courier-unicode.h>
 #include	"courierauth.h"
 
 
@@ -965,7 +965,7 @@ int callback_rc=0;
 
 			if (isnullname)
 				found_hier=mb_flags;
-			else 
+			else
 			{
 				strcat(strcat(strcpy(hiersepbuf, "\""),
 					      hierchs), "\"");
@@ -1006,19 +1006,13 @@ static void match_mailbox_prep(char *name)
 
 	/* First component, INBOX, is case insensitive */
 
-	if (
-#if	HAVE_STRNCASECMP
-	    strncasecmp(name, INBOX, sizeof(INBOX)-1) == 0
-#else
-	    strnicmp(name, INBOX, sizeof(INBOX)-1) == 0
-#endif
-	    )
+	if (strncasecmp(name, INBOX, sizeof(INBOX)-1) == 0)
 		for (i=0; name[i] && name[i] != HIERCH; i++)
 			name[i]=toupper( (int)(unsigned char)name[i] );
 
 	/* ... except that "shared" should be lowercase ... */
 
-	if (memcmp(name, "SHARED", 6) == 0)
+	if (strncmp(name, "SHARED", 6) == 0)
 		memcpy(name, "shared", 6);
 }
 
