@@ -61,6 +61,8 @@
 #define LISTVERSION 3
 
 extern void pop3dcapa();
+extern void pop3dlang(const char *);
+
 static void acctout(const char *disc);
 void rfc2045_error(const char *p)
 {
@@ -558,9 +560,10 @@ static void cleanup()
 			}
 
 			if (log_deletions)
-				fprintf(stderr, "INFO: DELETED, user=%s, ip=[%s], filename=%s\n",
+				fprintf(stderr, "INFO: DELETED, user=%s, ip=[%s], port=[%s], filename=%s\n",
 					getenv("AUTHENTICATED"),
-					getenv("TCPREMOTEIP"),
+					remoteip,
+					remoteport,
 					msglist_a[i]->filename);
 
 			if (unlink(msglist_a[i]->filename))
@@ -1009,6 +1012,12 @@ int	c;
 			continue;
 		}
 
+		if (strcmp(p, "LANG") == 0)
+		{
+			pop3dlang(strtok(NULL, "\r\n"));
+			continue;
+		}
+
 		if (strcmp(p, "DELE") == 0)
 		{
 		unsigned	i;
