@@ -1,5 +1,5 @@
 /*
-** Copyright 1998 - 2013 Double Precision, Inc.
+** Copyright 1998 - 2020 Double Precision, Inc.
 ** See COPYING for distribution information.
 */
 
@@ -568,6 +568,18 @@ static int mksockets()
 	return (0);
 }
 
+static int dup_and_check(int orig)
+{
+	int fd=sox_dup(orig);
+
+	if (fd < 0)
+	{
+		perror("dup");
+		exit(1);
+	}
+	return fd;
+}
+
 static int init(int argc, char **argv)
 {
 int	argn;
@@ -578,7 +590,7 @@ gid_t	gid=0;
 const	char *servname;
 int	forced=0;
 int	lockfd=-1;
- 
+
 	argn=argparse(argc, argv, arginfo);
 
 	if ((stoparg || restartarg) && pidarg == 0)
@@ -746,16 +758,17 @@ int	lockfd=-1;
 		{
 			signal(SIGHUP, SIG_IGN);
 			sox_close(0);
-			sox_dup(pipefd[0]);
+			dup_and_check(pipefd[0]);
 			sox_close(pipefd[0]);
 			sox_close(pipefd[1]);
 			sox_close(1);
 			open("/dev/null", O_WRONLY);
 			sox_close(2);
-			sox_dup(1);
+			dup_and_check(1);
 			closeaccess();
 			while ((p=fork()) == -1)
 			{
+				perror("fork");
 				sleep(5);
 			}
 			if (p == 0)
@@ -776,7 +789,7 @@ int	lockfd=-1;
 			_exit(0);
 		}
 		sox_close(2);
-		sox_dup(pipefd[1]);
+		dup_and_check(pipefd[1]);
 		sox_close(pipefd[0]);
 		sox_close(pipefd[1]);
 		while (wait(&waitstat) != p)
@@ -792,7 +805,7 @@ int	lockfd=-1;
 			return (-1);
 		}
 		sox_close(2);
-		sox_dup(fd);
+		dup_and_check(fd);
 		sox_close(fd);
 	}
 
@@ -1130,7 +1143,7 @@ static int doit(int argn, int argc, char **argv)
 		if (select(maxfd+1, &fdr, NULL, NULL, NULL) <= 0)
 		{
 			if (errno != EINTR)
-				perror("accept");
+				perror("select");
 			continue;
 		}
 
@@ -1149,7 +1162,7 @@ static int doit(int argn, int argc, char **argv)
 
 			if (pi->fd2 >= 0 && FD_ISSET(pi->fd2, &fdr) &&
 			    ((n=getfreeslot(&pidptr)),
-			     (sinl = sizeof(sin)), 
+			     (sinl = sizeof(sin)),
 			     (sockfd=sox_accept(pi->fd2,
 						(struct sockaddr *)&sin,
 						&sinl))) >= 0)
@@ -1259,7 +1272,7 @@ static void accepted(int n, int sockfd, RFC1035_NETADDR *sin, int sinl,
 		RFC1035_ADDR laddr;
 		int	lport;
 		socklen_t	i=sizeof(lsin);
-		
+
 			if (sox_getsockname(sockfd, (struct sockaddr *)&lsin, &i) == 0 &&
 				rfc1035_sockaddrip(&lsin, i, &laddr) == 0 &&
 				rfc1035_sockaddrport(&lsin, i, &lport) == 0 &&
@@ -1590,9 +1603,15 @@ static void docheckblocklist(struct blocklist_s *p, const char *nameptr)
 	struct rfc1035_res res;
 	unsigned int i;
 	int found;
+	unsigned char query_A[] = {RFC1035_TYPE_A, 0},
+		query_A_and_TXT[] = {RFC1035_TYPE_A, RFC1035_TYPE_TXT, 0};
+	size_t l=strlen(nameptr);
 
-	hostname[0]=0;
-	strncat(hostname, nameptr, RFC1035_MAXNAMESIZE);
+	if (l > RFC1035_MAXNAMESIZE)
+		l=RFC1035_MAXNAMESIZE;
+
+	memcpy(hostname, nameptr, l);
+	hostname[l]=0;
 
 	if (!varname)	varname="BLOCK";
 
@@ -1611,11 +1630,11 @@ static void docheckblocklist(struct blocklist_s *p, const char *nameptr)
 	if (p->allow)
 		wanttxt = p->msg != 0;
 	else
-		wanttxt = (p->msg == 0 || *p->msg == 0);
+		wanttxt = p->msg && strcmp(p->msg, "*") == 0;
 
-	(void)rfc1035_resolve_cname(&res,
+	(void)rfc1035_resolve_cname_multiple(&res,
 			hostname,
-			wanttxt ? RFC1035_TYPE_ANY:RFC1035_TYPE_A,
+			wanttxt ? query_A_and_TXT:query_A,
 			RFC1035_CLASS_IN, &replyp, 0);
 
 	if (!replyp)
@@ -1657,15 +1676,28 @@ static void docheckblocklist(struct blocklist_s *p, const char *nameptr)
 		if (p->allow)
 			set_allow_variable(varname, p->msg);
 
-		if (!search_txt_records(&res, p->allow, varname, replyp,
-					hostname) && !p->allow)
+		if ((replyp->next == NULL ||
+		     !search_txt_records(&res, p->allow, varname, replyp->next,
+					    hostname)) && !p->allow)
 		{
+			size_t l=strlen(p->zone)+40;
+			char *buf=malloc(l+1);
+
+			if (!buf)
+			{
+				perror("malloc");
+				_exit(1);
+			}
+
+			buf[snprintf(buf, l, "Blacklisted by %s", p->zone)]=0;
+
 			/*
 			** Even though we did not find a TXT record, we're here
 			** because of an A record, so for -blocks, we must
 			** set varname to something.
 			*/
-			mysetenv(varname, "Access denied.");
+			mkmymsg(varname, buf);
+			free(buf);
 		}
 
 		found=1;
@@ -1677,9 +1709,9 @@ static void docheckblocklist(struct blocklist_s *p, const char *nameptr)
 	** for a specific IP address, then take what we've got.
 	*/
 
-	if (p->ia.s_addr == INADDR_ANY && !found)
+	if (p->ia.s_addr == INADDR_ANY && !found && replyp->next)
 	{
-		if (search_txt_records(&res, p->allow, varname, replyp,
+		if (search_txt_records(&res, p->allow, varname, replyp->next,
 				       hostname))
 		{
 			/*
@@ -1903,13 +1935,13 @@ const char *p;
 	check_drop(fd);
 	sox_close(0);
 	sox_close(1);
-	sox_dup(fd);
-	sox_dup(fd);
+	dup_and_check(fd);
+	dup_and_check(fd);
 	sox_close(fd);
 	if (stderrarg && strcmp(stderrarg, "socket") == 0)
 	{
 		sox_close(2);
-		sox_dup(1);
+		dup_and_check(1);
 	}
 	proxy();
 	signal(SIGPIPE, SIG_DFL);
@@ -2084,9 +2116,9 @@ struct proxybuf proxy_[3];
 		sox_close(1);
 		sox_close(2);
 		errno=EINVAL;
-		if (sox_dup(pipefd0[0]) != 0 ||
-			sox_dup(pipefd1[1]) != 1 ||
-			sox_dup(pipefd2[1]) != 2)
+		if (dup_and_check(pipefd0[0]) != 0 ||
+			dup_and_check(pipefd1[1]) != 1 ||
+			dup_and_check(pipefd2[1]) != 2)
 		{
 			perror("dup(app)");
 			exit(1);
